{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWButton.pas                                           }
{ Short description  : Button with some additional properties                  }
{ Author             : Carsten Butzek                                          }
{ E-Mail             : cabutzek@muenster.de                                    }
{ Internet           : http://www.delphifreeware.de                            }
{ Version            : 1.0                                                     }
{ Date last modified :                                                         }
{ Delphi versions    : 2,3,4,5                                                 }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWButton                                                }
{ Classes          :                                                           }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

unit DFWButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DFWSound;

const
      ComponentVersion='1.0';
      ComponentAuthor='Carsten Butzek';
      ComponentEMail='cabutzek@muenster.de';
      ComponentHomePage='http://www.delphifreeware.de';

type

  TDFWButton = class(TButton)
  private
    { Private-Deklarationen }
    fDFWMouseDownSound : TDFWSound;
    fDFWMouseUpSound : TDFWSound;
    fDFWMouseClickSound : TDFWSound;
    fURL : string;
    fGotoURLOnClick : boolean;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
  protected
    { Protected-Deklarationen }
     fOnMouseEnter : TNotifyEvent;
     fOnMouseLeave : TNotifyEvent;
     procedure CMMouseEnter(var Message:TMessage);message CM_MOUSEENTER;
     procedure CMMouseLeave(var Message:TMessage);message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Click; override;
  public
    { Public-Deklarationen }
    procedure GotoURL;
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property MouseDownSound: TDFWSound read fDFWMouseDownSound write fDFWMouseDownSound;
    property MouseUpSound: TDFWSound read fDFWMouseUpSound write fDFWMouseUpSound;
    property MouseClickSound: TDFWSound read fDFWMouseClickSound write fDFWMouseClickSound;
    property URL: string read fURL write fURL;
    property GotoURLOnClick: boolean read fGotoURLOnClick write fGotoURLOnClick default false;
    property OnMouseEnter : TNotifyEvent read fOnMouseEnter write fOnMouseEnter;
    property OnMouseLeave : TNotifyEvent read fOnMouseLeave write fOnMouseLeave;
  end;


implementation
uses ShellAPI;

procedure TDFWButton.GotoURL;
var TempString : array[0..255] of char;
begin
    if fURL<>'' then begin
       StrPCopy(TempString,fURL);
       ShellExecute(0, Nil, TempString, Nil, Nil, SW_NORMAL);
    end;
end;

procedure TDFWButton.Click;
var TempString : array[0..255] of char;
begin
     inherited Click;
     if MouseClickSound.EnableSound then MouseClickSound.Play;
     if fGotoURLOnClick and (fURL<>'') then begin
        StrPCopy(TempString,fURL);
        ShellExecute(0, Nil, TempString, Nil, Nil, SW_NORMAL);
     end;
end;

procedure TDFWButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
     if MouseDownSound.EnableSound then MouseDownSound.Play;
end;

procedure TDFWButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
     if MouseUpSound.EnableSound then MouseUpSound.Play;
end;

constructor TDFWButton.Create(Owner: TComponent);
begin
     inherited Create(Owner);
     fDFWMouseClickSound:=TDFWSound.create;
     fDFWMouseDownSound:=TDFWSound.create;
     fDFWMouseUpSound:=TDFWSound.create;
     fGotoURLOnClick:=false;
     fURL:='http://www.delphi-freeware.de';
end;

destructor TDFWButton.Destroy;
begin
     inherited Destroy;
     fDFWMouseUpSound.free;
     fDFWMouseDownSound.free;
     fDFWMouseClickSound.free;
end;

procedure TDFWButton.CmMouseEnter(var Message:TMessage);
begin
   inherited;
   if (Assigned(fOnMouseEnter)) then fOnMouseEnter(Self);
end;

procedure TDFWButton.CmMouseLeave(var Message:TMessage);
begin
   inherited;
   if (Assigned(fOnMouseLeave)) then fOnMouseLeave(Self);
end;

function TDFWButton.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWButton.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWButton.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWButton.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
