{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWSmoothShow.pas                                       }
{ Short description  :                                                         }
{ Author             : Kambiz R. Khojasteh                                     }
{ E-Mail             : khojasteh@www.dci.co.ir                                 }
{ Internet           : http://www.crosswinds.net/~khojasteh/                   }
{ Version            : 1.21                                                    }
{ Date last modified : February 03, 2000                                       }
{ Delphi versions    : working with Delphi 3, 4 and 5 only                     }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWSmoothShow                                            }
{ Classes          : TTestProperty                                             }
{ Types            : TMinLoc                                                   }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSmoothShow;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DsgnIntf;

const
      ComponentVersion='1.21';
      ComponentAuthor='Kambiz R. Khojasteh';
      ComponentEMail='khojasteh@www.dci.co.ir';
      ComponentHomePage='http://www.crosswinds.net/~khojasteh/';

type

{ TTestProperty }

  TTestProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

{ TDFWSmoothShow }

  TMinLoc = (mlCenter, mlTopLeft, mlTopRight, mlBottomLeft, mlBottomRight, mlCustom);

  TDFWSmoothShow = class(TComponent)
  private
    FDelay: Word;
    FColor: TColor;
    FBorderWidth: TBorderWidth;
    FEnabled: Boolean;
    FReverse: Boolean;
    FMinLocation: TMinLoc;
    FMinLeft: Integer;
    FMinTop: Integer;
    FMinWidth: Word;
    FMinHeight: Word;
    FTest: TTestProperty;
    FOnFinishing: TNotifyEvent;
    FOnFinish: TNotifyEvent;
    Window: TCustomForm;
    OldRgn: HRgn;
    Timer: TTimer;
    Shadow: TForm;
    ThisStep: Integer;
    FBusy: Boolean;
    StartRect: TRect;
    StopRect: TRect;
    DiffRect: TRect;
    function GetCustomBounds: TRect;
    procedure SetCustomBounds(Value: TRect);
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    function MinimizedRect: TRect;
    function CurrentRect(ThisStep: Word): TRect;
    procedure TimerFired(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Execute;
    property Busy: Boolean read FBusy;
    property CustomBounds: TRect read GetCustomBounds write SetCustomBounds;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Color: TColor read FColor write FColor default clBlack;
    property BorderWidth: TBorderWidth read FBorderWidth write FBorderWidth default 1;
    property Delay: Word read FDelay write FDelay default 50;
    property Enabled: Boolean read FEnabled write FEnabled default True;
    property MinLocation: TMinLoc read FMinLocation write FMinLocation default mlCenter;
    property MinLeft: Integer read FMinLeft write FMinLeft default 0;
    property MinTop: Integer read FMinTop write FMinTop default 0;
    property MinWidth: Word read FMinWidth write FMinWidth default 27;
    property MinHeight: Word read FMinHeight write FMinHeight default 27;
    property Reverse: Boolean read FReverse write FReverse default False;
    property Test: TTestProperty read FTest write FTest stored False;
    property OnFinishing: TNotifyEvent read FOnFinishing write FOnFinishing;
    property OnFinish: TNotifyEvent read FOnFinish write FOnFinish;
  end;

implementation

{ TDFWSmoothShow }

constructor TDFWSmoothShow.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDelay := 50;
  FColor := clBlack;
  FEnabled := True;
  FReverse := False;
  FMinLocation := mlCenter;
  FBorderWidth := 1;
  CustomBounds := Rect(0, 0, 27, 27);
  FBusy := False;
end;

function TDFWSmoothShow.GetCustomBounds: TRect;
begin
  SetRect(Result, MinLeft, MinTop, MinLeft + MinWidth, MinTop + MinHeight);
end;

procedure TDFWSmoothShow.SetCustomBounds(Value: TRect);
begin
  With Value do
  begin
    FMinLeft := Left;
    FMinTop := Top;
    FMinWidth := Right - Left;
    FMinHeight := Bottom - Top;
  end;
end;

function TDFWSmoothShow.MinimizedRect: TRect;
begin
  case MinLocation of
   {$IFNDEF D3}
    mlCenter:
      SetRect(Result, Screen.DesktopLeft + Screen.DesktopWidth div 2,
        Screen.DesktopTop + Screen.DesktopHeight div 2, 0, 0);
    mlTopLeft:
      SetRect(Result, Screen.DesktopLeft, Screen.DesktopTop, 0, 0);
    mlTopRight:
      SetRect(Result, Screen.DesktopLeft + Screen.DesktopWidth,
        Screen.DesktopTop, 0, 0);
    mlBottomLeft:
      SetRect(Result, Screen.DesktopLeft, Screen.DesktopTop +
        Screen.DesktopHeight, 0, 0);
    mlBottomRight:
      SetRect(Result, Screen.DesktopLeft + Screen.DesktopWidth,
        Screen.DesktopTop + Screen.DesktopHeight, 0, 0);
    {$ELSE}
    mlCenter:
      SetRect(Result, Screen.Width div 2, Screen.Height div 2, 0, 0);
    mlTopLeft:
      SetRect(Result, 0, 0, 0, 0);
    mlTopRight:
      SetRect(Result, Screen.Width, 0, 0, 0);
    mlBottomLeft:
      SetRect(Result, 0, Screen.Height, 0, 0);
    mlBottomRight:
      SetRect(Result, Screen.Width, Screen.Height, 0, 0);
    {$ENDIF}
  else
    SetRect(Result, MinLeft, MinTop, MinWidth, MinHeight);
  end;
end;

function TDFWSmoothShow.CurrentRect(ThisStep: Word): TRect;
begin
  Result.Left := StartRect.Left + (ThisStep * DiffRect.Left) div 100;
  Result.Top := StartRect.Top + (ThisStep * DiffRect.Top) div 100;
  Result.Right := StartRect.Right + (ThisStep * DiffRect.Right) div 100;
  Result.Bottom := StartRect.Bottom + (ThisStep * DiffRect.Bottom) div 100;
end;

procedure TDFWSmoothShow.Execute;
var
  Rgn: HRgn;
begin
  if not (Assigned(Owner) and (Owner is TCustomForm)) then
    raise Exception.Create(Name + ': Owner must be a form');
  if not FBusy and (Enabled or (csDesigning in ComponentState)) then
  begin
    FBusy := True;
    Window := TCustomForm(Owner);
    if Reverse then
    begin
      SetRect(StartRect, Window.Left, Window.Top, Window.Width, Window.Height);
      StopRect := MinimizedRect;
    end
    else
    begin
      StartRect := MinimizedRect;
      SetRect(StopRect, Window.Left, Window.Top, Window.Width, Window.Height);
    end;
    DiffRect.Left := StopRect.Left - StartRect.Left;
    DiffRect.Top := StopRect.Top - StartRect.Top;
    DiffRect.Right := StopRect.Right - StartRect.Right;
    DiffRect.Bottom := StopRect.Bottom - StartRect.Bottom;
    Shadow := TForm.Create(Application);
    Shadow.BorderStyle := bsNone;
    Shadow.Color := Color;
    OldRgn := 0;
    GetWindowRgn(Window.Handle, OldRgn);
    Rgn := CreateRectRgn(0, 0, 0, 0);
    SetWindowRgn(Window.Handle, Rgn, True);
    ThisStep := 0;
    Timer := TTimer.Create(Self);
    Timer.Interval := Delay;
    Timer.OnTimer := TimerFired;
    TimerFired(Self);
  end;
end;

procedure TDFWSmoothShow.TimerFired(Sender: TObject);
var
  Rgn1, Rgn2: HRgn;
begin
  with CurrentRect(ThisStep) do Shadow.SetBounds(Left, Top, Right, Bottom);
  Rgn1 := CreateRectRgn(0, 0, Shadow.Width, Shadow.Height);
  Rgn2 := CreateRectRgn(BorderWidth, BorderWidth, Shadow.Width-BorderWidth, Shadow.Height-BorderWidth);
  CombineRgn(Rgn1, Rgn1, Rgn2, RGN_DIFF);
  DeleteObject(Rgn2);
  SetWindowRgn(Shadow.Handle, Rgn1, True);
  if not Shadow.Visible then Shadow.Show;
  Shadow.Update;
  if ThisStep >= 100 then
  begin
    Timer.Free;
    if not (csDesigning in ComponentState) and Assigned(OnFinishing) then
      OnFinishing(Self);
    Shadow.Free;
    SetWindowRgn(Window.Handle, OldRgn, True);
    Window.Update;
    FBusy := False;
    if not (csDesigning in ComponentState) and Assigned(OnFinish) then
      OnFinish(Self);
  end;
  ThisStep := (3 * ThisStep div 2) + 2;
  if ThisStep > 100 then ThisStep := 100;
end;

{ TTestProperty }

procedure TTestProperty.Edit;
begin
  TDFWSmoothShow(GetComponent(0)).Execute;
end;

function TTestProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TTestProperty.GetValue: string;
begin
  Result := '(DblClick to test)';
end;


function TDFWSmoothShow.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSmoothShow.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSmoothShow.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSmoothShow.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
