{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWSound.pas                                            }
{ Short description  : Sound types and procedures                              }
{ Author             : Carsten Butzek                                          }
{ E-Mail             : cabutzek@muenster.de                                    }
{ Internet           : http://www.delphifreeware.de                            }
{ Version            : 1.0                                                     }
{ Date last modified : 04/07/2000                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWSoundInfo, TDFWSound                                  }
{ Types            : Str150, Str20, Str30                                      }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSound;

interface
uses classes;
type
  Str150 = String[150];
  Str20  = String[20];
  Str30  = String[30];

  TDFWSoundInfo = class (TPersistent)
  private
    { Private-Deklarationen }
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    fSelectedIndex : Byte;
    fFileName      : str150;
    fRelativePath  : boolean;
    fResourceID    : str20;
    fWindowsSound  : str30;
  published
    { Published-Deklarationen }
    property SelectedIndex:Byte read fSelectedIndex write fSelectedIndex;
    property FileName:str150 read fFileName write fFileName;
    property RelativePath:boolean read fRelativePath write fRelativePath;
    property ResourceID:str20 read fResourceID write fResourceID;
    property WindowsSound:str30 read fWindowsSound write fWindowsSound;
  end;

  TDFWSound=class (TPersistent)
  private
    { Private-Deklarationen }
    fEnableSound:boolean;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    fSoundInfo: TDFWSoundInfo;
    procedure Play;
    constructor Create;
    destructor destroy;
  published
    { Published-Deklarationen }
    property EnableSound:boolean read fEnableSound write fEnableSound default false;
    property Sound: TDFWSoundInfo read fSoundInfo write fSoundInfo;
  end;

implementation

uses MMSystem, Sysutils;

procedure TDFWSound.Play;
var PC : array[0..30] of Char;
begin
     if Sound.SelectedIndex <>0 then PlaySound(nil, HInstance, SND_PURGE);
     case Sound.SelectedIndex of
          1: if FileExists (Sound.FileName) then begin
                strPCopy(PC, Sound.Filename);
                PlaySound(PC, HInstance,SND_FILENAME or SND_ASYNC);
             end;
          2: if Sound.ResourceID<>'' then begin
                strPCopy(PC, Sound.ResourceID);
                PlaySound(PC, HInstance, SND_RESOURCE or SND_ASYNC);
             end;
          3: if Sound.WindowsSound<>'' then begin
                strPCopy(PC, Sound.WindowsSound);
                PlaySound(PC, HInstance, SND_ASYNC);
             end;
     end;
end;

constructor TDFWSound.Create;
begin
     inherited Create;
     fEnableSound:=false;
     fSoundInfo:=TDFWSoundInfo.create;
end;

destructor TDFWSound.destroy;
begin
     fSoundInfo.free;
end;

end.
